/*
 * $Id: BarrageManager.java,v 1.3 2002/10/01 09:41:03 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */
package jp.gr.java_conf.abagames.noiz2;

import jp.gr.java_conf.abagames.bulletmlkxml.*;
import java.io.*;
import java.util.*;

/**
 * Barrage pattern manager.
 *
 * @version $Revision: 1.3 $
 */
public class BarrageManager {
  private Barrage[][] barragePattern, barrageQueue;

  /*
   * Types of enemies.
   * ('zako' - small enemies, 'middle' - middle class enemies, 'boss' - boss type enemies.)
   */
  private final int BARRAGE_TYPE_NUM = 3;

  private final String[] BARRAGE_DIR_NAME = {
    "zako", "middle", "boss",
  };

  private final int BARRAGE_MAX = 12;

  private Barrage[] barrage = new Barrage[BARRAGE_MAX];

  private void init() {
    barrageQueue = new Barrage[BARRAGE_TYPE_NUM][];
    barragePattern = new Barrage[BARRAGE_TYPE_NUM][];
    for ( int i=0 ; i<QUICK_APP_NUM ; i++ ) {
      pax[i] = -1;
    }
  }

  /**
   * Load the files of BulletML.
   */
  public BarrageManager()
   throws FileNotFoundException, IOException {
    init();
    for ( int i=0 ; i<BARRAGE_TYPE_NUM ; i++ ) {
      File dir = new File(BARRAGE_DIR_NAME[i] + File.separator);
      String[] fileName = dir.list();
      int fc = 0;
      for ( int j=0 ; j<fileName.length ; j++ ) {
        if ( !fileName[j].endsWith(".xml") ) continue;
        fc++;
      }
      barrageQueue[i] = new Barrage[fc];
      barragePattern[i] = new Barrage[fc];
      fc = 0;
      for ( int j=0 ; j<fileName.length ; j++ ) {
        if ( !fileName[j].endsWith(".xml") ) continue;

        System.out.println(fileName[j]);

        BulletmlManager manager = new BulletmlManager();
        manager.init();
        FileInputStream is = new FileInputStream(
         BARRAGE_DIR_NAME[i] + File.separator + fileName[j]);
        new Bulletml(is, manager);
        is.close();
        barragePattern[i][fc] = new Barrage(manager);
        barragePattern[i][fc].type = i;
        fc++;
      }

      System.out.println("----");
    }
  }

  private final String FILE_LIST_FILE_NAME = "list.txt";

  /**
   * Load the resources of BulletML from a JAR file.
   */
  public BarrageManager(Class appletClass)
   throws FileNotFoundException, IOException {
    init();
    for ( int i=0 ; i<BARRAGE_TYPE_NUM ; i++ ) {

      BufferedReader br = new BufferedReader(new InputStreamReader(
       appletClass.getResourceAsStream(
        BARRAGE_DIR_NAME[i] + "/" + FILE_LIST_FILE_NAME)));
      Vector fnl = new Vector();
      for ( ; ; ) {
        String fn = br.readLine();
        if ( fn == null ) break;
        if ( fn.endsWith(".xml") ) {
          fnl.addElement(fn);
        }
      }
      String[] fileName = new String[fnl.size()];
      fnl.copyInto(fileName);
      barrageQueue[i] = new Barrage[fileName.length];
      barragePattern[i] = new Barrage[fileName.length];
      for ( int j=0 ; j<fileName.length ; j++ ) {

        System.out.println(fileName[j]);

        BulletmlManager manager = new BulletmlManager();
        manager.init();
        InputStream is = appletClass.getResourceAsStream(
         BARRAGE_DIR_NAME[i] + "/" + fileName[j]);
        new Bulletml(is, manager);
        is.close();
        barragePattern[i][j] = new Barrage(manager);
        barragePattern[i][j].type = i;
      }

      System.out.println("----");
    }
  }

  private BulletmlPlayer player;

  public void setPlayer(BulletmlPlayer player) {
    this.player = player;
    for ( int i=0 ; i<BARRAGE_TYPE_NUM ; i++ ) {
      for ( int j=0 ; j<barragePattern[i].length ; j++ ) {
        barragePattern[i][j].manager.setPlayer(player);
      }
    }
  }

  // A Random instance for cotroling the enemies' pattern.
  private Random patRnd = new Random();

  private int stage, scene;
  private final float[][] stageSpec = {
    {0, 0.0f, 0.2f},
    {1, 0.3f, 0.4f}, {2, 1.0f, 0.3f},
    {3, 3.0f, 0.5f}, {4, 5.0f, 0.2f}, {5, 1.5f, 0.6f},
    {6, 4.0f, 1.5f}, {7, 7.0f, 0.7f}, {8, 6.0f, 1.2f}, {9, 12.0f, 0.2f},
    {-1, 1.0f, 0.4f}
  };

  private int sceneCnt;
  private float level, levelInc;
  private String sceneStr, sceneScoreStr;
  private boolean endless;

  private Random rnd = new Random();

  public void initStage(int stage) {
    this.stage = stage;
    scene = 0;
    init((long)stageSpec[stage][0], stageSpec[stage][1], stageSpec[stage][2]);
    sceneStr = "";
    ship.init();
  }

  public void initStageAsDemo(int stage) {
    initStage(stage);
    setBarrages(level, false, false);
    sceneCnt = Integer.MAX_VALUE;
  }

  public void init(long seed, float startLevel, float levelInc) {
    // Init a barrage queue.
    for ( int i=0 ; i<BARRAGE_TYPE_NUM ; i++ ) {
      for ( int j=0 ; j<barragePattern[i].length ; j++ ) {
        barrageQueue[i][j] = barragePattern[i][j];
        barragePattern[i][j].manager.reinit();
      }
    }

    // Shuffle.
    if ( seed >= 0 ) {
      patRnd.setSeed(seed);
      endless = false;
    } else {
      patRnd.setSeed(rnd.nextLong());
      endless = true;
    }
    for ( int i=0 ; i<BARRAGE_TYPE_NUM ; i++ ) {
      int bn = barrageQueue[i].length;
      for ( int j=0 ; j<64 ; j++ ) {
        int n1 = Math.abs(patRnd.nextInt())%bn, n2 = Math.abs(patRnd.nextInt())%bn;
        Barrage tb = barrageQueue[i][n1];
        barrageQueue[i][n1] = barrageQueue[i][n2];
        barrageQueue[i][n2] = tb;
      }
      for ( int j=0 ; j<bn ; j++ ) {
        barrageQueue[i][j].maxRank = (float)(Math.abs(patRnd.nextInt())%70)/100 + 0.3f;
      }
    }

    sceneCnt = 0;
    barrageNum = 0;
    level = startLevel;
    this.levelInc = levelInc;
  }

  /**
   * Roll the barrage queue after the new barrage pattern is set.
   */
  private void rollBarragePattern(Barrage[] br) {
    int n = (int)((float)br.length/((float)(Math.abs(patRnd.nextInt())%32)/32+1));
    if ( n == 0 ) return;
    Barrage tbr = br[0];
    for ( int i=0 ; i<n-1 ; i++ ) {
      br[i] = br[i+1];
    }
    br[n-1] = tbr;
    br[0].maxRank *= 2;
    while ( br[0].maxRank > 1 ) br[0].maxRank -= 0.7f;
  }

  private int barrageNum;
  private boolean bossMode;

  /**
   * Make the barrage pattern of this scene.
   */
  public void setBarrages(float level, boolean bossMode, boolean midMode) {
    this.bossMode = bossMode;

    int bpn = 0;
    barrageNum = 0;
    for ( int bn = 0 ; bn < BARRAGE_MAX ; bn++ ) {
      if ( bossMode ) {
        if ( bn == 0 ) bpn = 0;
        else bpn = 2;
      }
      barrageNum++;
      rollBarragePattern(barrageQueue[bpn]);
      if ( level < barrageQueue[bpn][0].maxRank ) {
        if ( level < 0 ) level = 0;
        barrage[bn] = barrageQueue[bpn][0];
        barrage[bn].manager.setRank(level);
        break;
      }
      barrage[bn] = barrageQueue[bpn][0];
      barrage[bn].manager.setRank(barrageQueue[bpn][0].maxRank);
      if ( !bossMode ) {
        //level /= 2+(barrageQueue[bpn][0].maxRank*2);
        level -= 1+barrageQueue[bpn][0].maxRank;
      } else {
        //if ( bn > 0 ) level /= 8+(barrageQueue[bpn][0].maxRank*12);
        if ( bn > 0 ) level -= 4+(barrageQueue[bpn][0].maxRank*6);
      }

      if ( midMode ) {
        bpn = BARRAGE_TYPE_NUM-1;
      } else {
        bpn++;
        if ( bpn >= BARRAGE_TYPE_NUM ) bpn = 0;
      }
    }

    pax[0] = (Math.abs(patRnd.nextInt())%(Screen.WIDTH*2/3) + (Screen.WIDTH/6))<<8;
    pay[0] = (Math.abs(patRnd.nextInt())%(Screen.HEIGHT/6) + (Screen.HEIGHT/10))<<8;
    for ( int i=1 ; i<QUICK_APP_NUM ; i++ ) pax[i] = -1;

    scene++;
    if ( !endless ) {
      sceneStr = (stage+1) + "-" + scene;
    } else {
      sceneStr = Integer.toString(scene);
    }
  }

  private final int QUICK_APP_NUM = 1;
  private int[] enNum = new int[QUICK_APP_NUM];

  public static final int INTERVAL_BASE = 16;
  private final int PROCESS_SPEED_DOWN_BULLETS_NUM = 80;
  private int bulletNum;

  public void moveBullets() {
    for ( int i=0 ; i<barrageNum ; i++ ) {
      barrage[i].manager.setShipPos(ship.x, ship.y, ship.px, ship.py);
    }
    bulletNum = 0;
    for ( int i=0 ; i<QUICK_APP_NUM ; i++ ) {
      enNum[i] = 0;
    }
    for ( int i=0 ; i<barrageNum ; i++ ) {
      barrage[i].manager.moveBullets();
      bulletNum += barrage[i].manager.getBulletsNum();
      for ( int j=0 ; j<QUICK_APP_NUM ; j++ ) {
        enNum[j] += barrage[i].manager.getEnemiesNum(j);
      }
    }
    // A game speed becomes slow as many bullets appears.
    int itv = INTERVAL_BASE;
    if ( bulletNum > PROCESS_SPEED_DOWN_BULLETS_NUM ) {
      itv += (bulletNum-PROCESS_SPEED_DOWN_BULLETS_NUM) * INTERVAL_BASE /
             PROCESS_SPEED_DOWN_BULLETS_NUM;
      if ( itv > INTERVAL_BASE*2 ) itv = INTERVAL_BASE*2;
    }
    player.setInterval(itv);
  }

  public void drawBullets() {
    for ( int i=0 ; i<barrageNum ; i++ ) {
      barrage[i].manager.drawBullets();
    }
  }

  public void clearZakoBullets() {
    for ( int i=0 ; i<barrageNum ; i++ ) {
      barrage[i].manager.clearZako(ship);
    }
  }

  public void clearBullets() {
    for ( int i=0 ; i<barrageNum ; i++ ) {
      barrage[i].manager.clear(ship);
    }
  }

  private final int SCENE_TERM = 1000;
  private final int SCENE_END_TERM = 100;
  private final int SCENE_DISP_TERM = 172;
  private int sceneDispCnt;
  private final int ZAKO_APP_TERM = 1500;
  private int zakoAppCnt;

  private final int[] appFreq = {90, 360, 800};
  private final int[] shield = {1, 2, 3};

  private int[] pax = new int[QUICK_APP_NUM];
  private int[] pay = new int[QUICK_APP_NUM];

  /**
   * Add enemies.
   */
  public void addBullets() {
    // Scene time control.
    sceneCnt--;
    if ( sceneDispCnt > 0 ) sceneDispCnt--;
    if ( sceneCnt < 0 ) {
      if ( !endless && scene > 0 ) {
        int ss = ship.getSceneScore();
        int ssof = ship.clearScene(stage, scene-1);
        if ( ssof < 0 ) {
          sceneScoreStr = ss + " " + ssof;
        } else {
          sceneScoreStr = ss + " +" + ssof;
        }
      } else {
        sceneScoreStr = "";
      }
      clearBullets();
      if ( !endless && scene >= 10 ) {
        sceneStr = "";
        sceneDispCnt = SCENE_DISP_TERM;
        ship.addLeftBonus();
        player.clearStage();
        return;
      }
      if ( scene%10 == 9 ) {
        sceneCnt = Integer.MAX_VALUE;
        zakoAppCnt = ZAKO_APP_TERM;
        setBarrages(level, true, false);
        addBossBullet();
      } else {
        sceneCnt = SCENE_TERM;
        if ( scene%10 == 4 ) {
          setBarrages(level, false, true);
        } else {
          setBarrages(level, false, false);
        }
      }
      level += levelInc;
      sceneDispCnt = SCENE_DISP_TERM;
    }
    if ( sceneCnt < SCENE_END_TERM ) return;
    if ( bossMode ) {
      if ( bossBullet.x == BulletImpl.NOT_EXIST ) {
        sceneCnt = 0;
      }
    }

    int x, y;
    for ( int i=0 ; i<barrageNum ; i++ ) {
      if ( bossMode ) {
        if ( i > 0 ) break;
        if ( zakoAppCnt <= 0 ) break;
        zakoAppCnt--;
      }
      int type = barrage[i].type;
      int frq = appFreq[type];
      // An additional enemy appears when there is no enemy of the same type.
      if ( type < QUICK_APP_NUM && enNum[type] == 0 && pax[type] != -1 ) {
        x = pax[type]; y = pay[type];
        barrage[i].manager.addTopBullet(x, y, 180, shield[type], type);
        enNum[type]++;
        break;
      }

      if ( (Math.abs(patRnd.nextInt())%frq) == 0 ) {
        x = Math.abs(patRnd.nextInt())%(BulletmlManager.screenWidth*2/3) +
            (BulletmlManager.screenWidth/6);
        y = Math.abs(patRnd.nextInt())%(BulletmlManager.screenHeight/6) +
            (BulletmlManager.screenHeight/10);
        if ( type < QUICK_APP_NUM ) {
          pax[type] = x; pay[type] = y;
        }
        barrage[i].manager.addTopBullet(x, y, 180, shield[type], type);
      }
    }
  }

  public static final int BOSS_TYPE = 3;
  private final int BOSS_SHIELD = 24;
  private BulletImpl bossBullet;

  public void addBossBullet() {
    bossBullet = null;
    for ( int i=0 ; i<barrageNum ; i++ ) {
      if ( barrage[i].type != 2 ) continue;
      BulletImpl bl = barrage[i].manager.addTopBullet(
       BulletmlManager.screenWidth/2, BulletmlManager.screenHeight/5, 180, BOSS_SHIELD, BOSS_TYPE);
      if ( bossBullet == null ) {
        bossBullet = bl;
      } else {
        bl.parent = bossBullet;
      }
    }
  }

  private Ship ship;

  public void setShip(Ship ship) {
    this.ship = ship;
  }

  /**
   * Wipe bullets around the destoryed enemy.
   */
  public void wipeBullets(int x, int y, int width) {
    for ( int i=0 ; i<barrageNum ; i++ ) {
      barrage[i].manager.wipeBullets(ship, x, y, width);
    }
  }

  /**
   * Draw the scene indicator.
   */
  public void drawScene() {
    if ( sceneDispCnt <= 0 ) return;
    //LetterRender.drawString(sceneStr, 0, Screen.HEIGHT-27, 7, 0xffffdddd);
    //LetterRender.drawString(sceneScoreStr, 0, Screen.HEIGHT-45, 5, 0xffddffdd);
    LetterRender.drawString(sceneStr, 0, 24, 7, 0xffffdddd);
    LetterRender.drawString(sceneScoreStr, 0, 45, 5, 0xffddffdd);
  }
}
