/*
 * $Id: Noiz2.java,v 1.4 2002/10/04 14:05:13 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */
package jp.gr.java_conf.abagames.noiz2;

import jp.gr.java_conf.abagames.bulletmlkxml.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;

/**
 * Noiz2 bootstrap.
 *
 * @version $Revision: 1.4 $
 */
public class Noiz2 extends Applet {
  public static final String VERSION = "0.21";
  public static final int VERSION_NUM = 20;

  private Screen screen;
  private BarrageManager manager;
  private BulletmlPlayer player;
  private Ship ship;
  private AttractManager atMng;
  private PrefManager prMng;

  // Is this application executed as the application or the applet?
  private boolean standAlone = false;

  public void setStandalone(boolean f) {
    standAlone = f;
  }

  public void init() {
    try {
      prMng = new PrefManager();
      if ( standAlone ) {
        manager = new BarrageManager();
        prMng.setUseFile(true);
      } else {
        manager = new BarrageManager(this.getClass());
        prMng.setUseFile(false);
      }
      prMng.init();
      prMng.load();

      screen = new Screen();
      player = new BulletmlPlayer();
      ship = new Ship(manager, player, prMng);
      manager.setShip(ship);
      atMng = new AttractManager(manager, screen, ship, prMng);
      if ( standAlone ) {
        atMng.loadImages();
      } else {
        atMng.loadImages(this);
      }
      player.init(manager, screen, ship, atMng);
      LetterRender.setScreen(screen);
    } catch ( Exception e ) {
      e.printStackTrace();
      manager = null;
    }
  }

  private void deactiveGame() {
    if ( player != null ) {
      player.deactive();
    }
  }

  public void start() {
    ship.init();
    atMng.initTitle();
    firstRepaint = true;
    repaint();
  }

  public void stop() {
    deactiveGame();
  }

  public void destroy() {
    deactiveGame();
  }

  private boolean firstRepaint = false;

  public void paint(Graphics g) {
    if ( screen != null ) {
      screen.makeBuffer(this);
      if ( manager != null && firstRepaint ) {
        firstRepaint = false;
        player.start();
      }
    }
  }

  public void update(Graphics g) {
    paint(g);
  }

  public boolean handleEvent(Event event) {
    if ( manager != null && event.target == this ) {
      switch ( event.id ) {
      case Event.MOUSE_MOVE:
        switch ( atMng.state ) {
        case AttractManager.TITLE:
          atMng.setPos(event.x, event.y);
          break;
        case AttractManager.IN_GAME:
        case AttractManager.STAGE_CLEAR:
          ship.setPos(event.x<<8, event.y<<8);
          break;
        }
        return true;
      case Event.MOUSE_DOWN:
        atMng.mousePressed();
        return true;
      }
    }
    return false;
  }

  private static void usage() {
    System.err.println(
     "Usage: noiz2 [-fullscreen]");
  }

  private static boolean fullScreen = false;

  private static void parseArgs(String[] args) throws IOException {
    for ( int i=0 ; i<args.length ; i++ ) {
      if ( args[i].equals("-fullscreen") ) {
        fullScreen = true;
      } else if ( args[i].startsWith("-") )  {
        throw new IOException("Invalid args.");
      }
    }
  }

  public static void main(String[] args) {
    /*try {
      parseArgs(args);
    } catch ( IOException e ) {
      usage();
      System.exit(1);
    }*/
    Noiz2 noiz2 = new Noiz2();
    noiz2.setStandalone(true);

    /*GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
    final GraphicsDevice device = ge.getDefaultScreenDevice();
    GraphicsConfiguration gc = device.getDefaultConfiguration();
    DisplayMode mode = new DisplayMode(640, 480, 32, DisplayMode.REFRESH_RATE_UNKNOWN);*/

    Frame frame;
    //frame = new Frame(gc) {
    frame = new Frame() {
      protected void processWindowEvent(WindowEvent e) {
        super.processWindowEvent(e);
        if (e.getID() == WindowEvent.WINDOW_CLOSING) {
          //device.setFullScreenWindow(null);
          System.exit(0);
        }
      }
      public synchronized void setTitle(String title) {
        super.setTitle(title);
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
      }
    };
    /*if ( fullScreen ) {
      frame.setUndecorated(true);
      device.setFullScreenWindow(frame);
      device.setDisplayMode(mode);
    }*/

    frame.setTitle("Noiz2");
    frame.add(noiz2, BorderLayout.CENTER);
    noiz2.init();
    noiz2.start();
    if ( !fullScreen ) {
      frame.setSize(Screen.WIDTH, Screen.HEIGHT + 24);
      Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
      frame.setLocation((d.width - frame.getSize().width) / 2,
                      (d.height - frame.getSize().height) / 2);
    }
    frame.setVisible(true);
  }
}
