/*
 * $Id: PrefManager.java,v 1.3 2002/10/01 09:41:04 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */
package jp.gr.java_conf.abagames.noiz2;

import java.io.*;

/**
 * Save/load preferences(hiscore).
 *
 * @version $Revision: 1.3 $
 */
public class PrefManager {

  private int[] stageScore
   = new int[AttractManager.STAGE_NUM];
  private int[][] sceneScore
   = new int[AttractManager.STAGE_NUM-1][AttractManager.SCENE_NUM];
  private boolean[] stageOpened = new boolean[AttractManager.STAGE_NUM];
  private boolean[] stageCleared = new boolean[AttractManager.STAGE_NUM];

  private boolean useFile;

  public void setUseFile(boolean f) {
    useFile = f;
  }

  public void init() {
    for ( int i=0 ; i<AttractManager.STAGE_NUM ; i++ ) {
      stageScore[i] = 10000;
      //if ( i == 0 || !useFile ) stageOpened[i] = true;
      stageOpened[i] = true;
      stageCleared[i] = false;
      if ( i == AttractManager.STAGE_NUM-1 ) continue;
      for ( int j=0 ; j<AttractManager.SCENE_NUM ; j++ ) {
        sceneScore[i][j] = 1000;
      }
    }
  }

  private final String PREF_FILE_NAME = "noiz2.prf";

  public void save() {
    if ( !useFile ) return;
    try {
      DataOutputStream os =
       new DataOutputStream(new FileOutputStream(PREF_FILE_NAME));
      os.writeInt(Noiz2.VERSION_NUM);
      for ( int i=0 ; i<AttractManager.STAGE_NUM ; i++ ) {
        os.writeInt(stageScore[i]);
        os.writeBoolean(stageOpened[i]);
        os.writeBoolean(stageCleared[i]);
        if ( i == AttractManager.STAGE_NUM-1 ) continue;
        for ( int j=0 ; j<AttractManager.SCENE_NUM ; j++ ) {
          os.writeInt(sceneScore[i][j]);
        }
      }
      os.close();
    } catch ( Exception e ) { }
  }

  public void load() {
    if ( !useFile ) return;
    try {
      DataInputStream is =
       new DataInputStream(new FileInputStream(PREF_FILE_NAME));
      int ver = is.readInt();
      if ( ver != Noiz2.VERSION_NUM ) return;
      for ( int i=0 ; i<AttractManager.STAGE_NUM ; i++ ) {
        stageScore[i] = is.readInt();
        stageOpened[i] = is.readBoolean();
        stageCleared[i] = is.readBoolean();
        if ( i == AttractManager.STAGE_NUM-1 ) continue;
        for ( int j=0 ; j<AttractManager.SCENE_NUM ; j++ ) {
          sceneScore[i][j] = is.readInt();
        }
      }
      is.close();
    } catch ( Exception e ) { }
  }

  public int getStageScore(int stage) {
    return stageScore[stage];
  }

  public void setStageScore(int stage, int score) {
    stageScore[stage] = score;
  }

  public int clearScene(int ss, int stage, int scene) {
    int rss = sceneScore[stage][scene];
    int ssof = ss - rss;
    if ( ss > rss ) {
      sceneScore[stage][scene] = ss;
    }
    return ssof;
  }

  /*
   * Stages are locked until some stages are cleared.
   */
  private int[][] nextOpen = {
    {1,2},{3,4},{4,5},{6,7},{7,8},{8,9}
  };

  public void clearStage(int stage) {
    stageCleared[stage] = true;
    if ( stage < nextOpen.length ) {
      stageOpened[nextOpen[stage][0]] = true;
      stageOpened[nextOpen[stage][1]] = true;
    }
    for ( int i=0 ; i<AttractManager.STAGE_NUM-1 ; i++ ) {
      if ( !stageCleared[i] ) return;
    }
    stageOpened[AttractManager.STAGE_NUM-1] = true;
  }

  public boolean isOpened(int stage) {
    return stageOpened[stage];
  }
}
